package com.example.test007;

import java.util.Arrays;

import android.app.Activity;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.View;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

/**
 * (c) 2014 Toshio Sugihara.
 * ライセンスは、GNU GPL2です。
 * @author sugihara
 *
 */

public class MainActivity extends Activity {
  EditText edit1;
  TextView text2;
  long startTime;
  long endTime;

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.activity_main);

    edit1 = (EditText) findViewById(R.id.main_vnumedit);
    text2 = (TextView) findViewById(R.id.main_text2);
  }

  /**
   * バブルソート開始ボタンが押されたときの処理。
   * @param v 呼び出したView
   */
  public void clickStart(View v) {
    // データ件数を読み取る
    int n = 0;
    try {
      n = Integer.parseInt(edit1.getText().toString());
    } catch (Exception e) {
    }
    // 範囲が正しくない場合は抜ける
    if (n < 2 || n > 200000) {
      text2.setText(R.string.main_error1);
      Toast.makeText(this, R.string.main_error1, Toast.LENGTH_LONG).show();
      return;
    }
    // 計算をはじめる
    startTime = System.currentTimeMillis();
    text2.setText("バブルソート計算中\n個数：" + n + "\n開始：" + startTime);
    new BubbleTask().execute(n);
  }

  /**
   * クイックソート開始ボタンが押されたときの処理。
   * クイックソートを実行する。
   * @param v 呼び出したView
   */
  public void clickQuick(View v){
    // データ件数を読み取る
    int n = 0;
    try {
      n = Integer.parseInt(edit1.getText().toString());
    } catch (Exception e) {
    }
    // 範囲が正しくない場合は抜ける(2-16777216)
    if (n < 2 || n > 16777216) {
      text2.setText(R.string.main_error2);
      Toast.makeText(this, R.string.main_error2, Toast.LENGTH_LONG).show();
      return;
    }
    // 計算をはじめる
    startTime = System.currentTimeMillis();
    text2.setText("クイックソート計算中\n個数：" + n + "\n開始：" + startTime);
    new QuickTask().execute(n);
  }
  /**
   * バブルソートを行うためのワーカースレッドクラス。
   * バブルソートの実施と、結果の画面への反映を行う。
   * @author sugihara
   *
   */
  private class BubbleTask extends AsyncTask<Integer, Void, Integer> {

    /**
     * ソートの実行と合計値の算出と、検算を行う。
     * @param arg 要素は１つ。[0]にソートするデータ件数をセットする。
     */
    @Override
    protected Integer doInBackground(Integer... arg) {
      // 指定された個数の整数値に対しバブルソートを行う。
      // ソートが終わったら検算結果を返す。0:正常 1:異常
      /* 配列確保 */
      int size = arg[0];
      int[] a = new int[size];
      int i, j, tmp;
      long total;
      int check;

      /* 初期値代入 */
      for (i = 0; i < size; i++) {
        a[i] = size - i; /* 降順で最後が１ */
      }
      /* ソート */
      for (i = size - 1; i > 0; i--) {
        for (j = 0; j < i; j++) {
          if (a[j] > a[j + 1]) {
            tmp = a[j];
            a[j] = a[j + 1];
            a[j + 1] = tmp;
          }
        }
      }
      // 合計計算
      total = 0;
      for (i = 0; i < size; i++) {
        total += a[i];
        // System.out.println(a[i]); //debug
      }
      System.out.println("合計 = " + total);// デバッグ時のみ表示

      // 結果チェック
      check = 0;
      for (i = 0; i < size - 1; i++) {
        if (a[i] > a[i + 1]) {
          check = 1;
        }
      }
      System.out.println("結果チェック = " + check); // デバッグ時のみ表示
      // 検算結果を返す
      return check;
    }
    /**
     * メインスレッドで結果を画面出力する。
     * @param result 検算結果 0:正常 1:異常
     */
    protected void onPostExecute(Integer result) {
      // 終了時刻の取得
      endTime = System.currentTimeMillis();
      // 結果を表示する。
      text2.setText("バブルソート完了\n検算：" + result + "\n開始：" + startTime + "\n終了："
          + endTime + "\n所要：" + (endTime - startTime)+"\n時間の単位はミリ秒");
    }
  }
  
  /**
   * クイックソートによる並べ替えを実行するタスク
   * @author sugihara
   *
   */
  private class QuickTask extends AsyncTask<Integer, Void, Integer> {
    /**
     * @param arg 要素は1つ。先頭要素がソートするデータ件数
     * @return 検算結果 0:正常 1:異常
     */
    @Override
    protected Integer doInBackground(Integer... arg) {
      int size = arg[0]; // データ件数
      int [] a = new int[size]; // データ本体
      int i;
      long total;
      int check;
      /* 初期値代入 */
      for(i=0;i<size;i++){
        a[i] = size-i; /* 降順で最後が１ */
      }
      // 昇順に並び替え
      Arrays.sort(a);
      
      // 合計計算
      total = 0;
      for(i=0;i<size;i++){
        total += a[i];
        // printf("%d\n",a[i]);//debug
      }
      System.out.format("合計 = %d\n", total);

      // 結果チェック
      check = 0;
      for(i=0;i<size-1;i++){
        if(a[i]>a[i+1]){
          check = 1;
        }
      }
      System.out.format("結果チェック = %d\n", check);

      return check;
    }
    /**
     * メインスレッドでタスク終了後の後処理を実行。
     * 画面に処理結果を表示する。
     * @param result 処理結果 0:正常 1:異常
     */
    protected void onPostExecute(Integer result) {
      // 終了時刻の取得
      endTime = System.currentTimeMillis();
      // 結果を表示する。
      text2.setText("クイックソート完了\n検算：" + result + "\n開始：" + startTime + "\n終了："
          + endTime + "\n所要：" + (endTime - startTime)+"\n時間の単位はミリ秒");      
    }
  }
}
